# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class UserList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the UserList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.frontline_api.v1.user.UserList
        :rtype: twilio.rest.frontline_api.v1.user.UserList
        """
        super(UserList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, sid):
        """
        Constructs a UserContext

        :param sid: The SID of the User resource to fetch

        :returns: twilio.rest.frontline_api.v1.user.UserContext
        :rtype: twilio.rest.frontline_api.v1.user.UserContext
        """
        return UserContext(self._version, sid=sid, )

    def __call__(self, sid):
        """
        Constructs a UserContext

        :param sid: The SID of the User resource to fetch

        :returns: twilio.rest.frontline_api.v1.user.UserContext
        :rtype: twilio.rest.frontline_api.v1.user.UserContext
        """
        return UserContext(self._version, sid=sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FrontlineApi.V1.UserList>'


class UserPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the UserPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.frontline_api.v1.user.UserPage
        :rtype: twilio.rest.frontline_api.v1.user.UserPage
        """
        super(UserPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of UserInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.frontline_api.v1.user.UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        return UserInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FrontlineApi.V1.UserPage>'


class UserContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, sid):
        """
        Initialize the UserContext

        :param Version version: Version that contains the resource
        :param sid: The SID of the User resource to fetch

        :returns: twilio.rest.frontline_api.v1.user.UserContext
        :rtype: twilio.rest.frontline_api.v1.user.UserContext
        """
        super(UserContext, self).__init__(version)

        # Path Solution
        self._solution = {'sid': sid, }
        self._uri = '/Users/{sid}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the UserInstance

        :returns: The fetched UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return UserInstance(self._version, payload, sid=self._solution['sid'], )

    def update(self, friendly_name=values.unset, avatar=values.unset,
               state=values.unset, is_available=values.unset):
        """
        Update the UserInstance

        :param unicode friendly_name: The string that you assigned to describe the User
        :param unicode avatar: The avatar URL which will be shown in Frontline application
        :param UserInstance.StateType state: Current state of this user
        :param bool is_available: Whether the User is available for new conversations

        :returns: The updated UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        data = values.of({
            'FriendlyName': friendly_name,
            'Avatar': avatar,
            'State': state,
            'IsAvailable': is_available,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return UserInstance(self._version, payload, sid=self._solution['sid'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FrontlineApi.V1.UserContext {}>'.format(context)


class UserInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class StateType(object):
        ACTIVE = "active"
        DEACTIVATED = "deactivated"

    def __init__(self, version, payload, sid=None):
        """
        Initialize the UserInstance

        :returns: twilio.rest.frontline_api.v1.user.UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        super(UserInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'identity': payload.get('identity'),
            'friendly_name': payload.get('friendly_name'),
            'avatar': payload.get('avatar'),
            'state': payload.get('state'),
            'is_available': payload.get('is_available'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: UserContext for this UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserContext
        """
        if self._context is None:
            self._context = UserContext(self._version, sid=self._solution['sid'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def identity(self):
        """
        :returns: The string that identifies the resource's User
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def friendly_name(self):
        """
        :returns: The string that you assigned to describe the User
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def avatar(self):
        """
        :returns: The avatar URL which will be shown in Frontline application
        :rtype: unicode
        """
        return self._properties['avatar']

    @property
    def state(self):
        """
        :returns: Current state of this user
        :rtype: UserInstance.StateType
        """
        return self._properties['state']

    @property
    def is_available(self):
        """
        :returns: Whether the User is available for new conversations
        :rtype: bool
        """
        return self._properties['is_available']

    @property
    def url(self):
        """
        :returns: An absolute URL for this user.
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the UserInstance

        :returns: The fetched UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        return self._proxy.fetch()

    def update(self, friendly_name=values.unset, avatar=values.unset,
               state=values.unset, is_available=values.unset):
        """
        Update the UserInstance

        :param unicode friendly_name: The string that you assigned to describe the User
        :param unicode avatar: The avatar URL which will be shown in Frontline application
        :param UserInstance.StateType state: Current state of this user
        :param bool is_available: Whether the User is available for new conversations

        :returns: The updated UserInstance
        :rtype: twilio.rest.frontline_api.v1.user.UserInstance
        """
        return self._proxy.update(
            friendly_name=friendly_name,
            avatar=avatar,
            state=state,
            is_available=is_available,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FrontlineApi.V1.UserInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.chat.v3.channel import ChannelList


class V3(Version):

    def __init__(self, domain):
        """
        Initialize the V3 version of Chat

        :returns: V3 version of Chat
        :rtype: twilio.rest.chat.v3.V3.V3
        """
        super(V3, self).__init__(domain)
        self.version = 'v3'
        self._channels = None

    @property
    def channels(self):
        """
        :rtype: twilio.rest.chat.v3.channel.ChannelList
        """
        if self._channels is None:
            self._channels = ChannelList(self)
        return self._channels

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Chat.V3>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import serialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_invite import InteractionChannelInviteList
from twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant import InteractionChannelParticipantList


class InteractionChannelList(ListResource):

    def __init__(self, version, interaction_sid):
        """
        Initialize the InteractionChannelList

        :param Version version: Version that contains the resource
        :param interaction_sid: The unique string that identifies the resource.

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelList
        """
        super(InteractionChannelList, self).__init__(version)

        # Path Solution
        self._solution = {'interaction_sid': interaction_sid, }
        self._uri = '/Interactions/{interaction_sid}/Channels'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams InteractionChannelInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists InteractionChannelInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of InteractionChannelInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return InteractionChannelPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of InteractionChannelInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return InteractionChannelPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a InteractionChannelContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        """
        return InteractionChannelContext(
            self._version,
            interaction_sid=self._solution['interaction_sid'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a InteractionChannelContext

        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        """
        return InteractionChannelContext(
            self._version,
            interaction_sid=self._solution['interaction_sid'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionChannelList>'


class InteractionChannelPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the InteractionChannelPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param interaction_sid: The unique string that identifies the resource.

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelPage
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelPage
        """
        super(InteractionChannelPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of InteractionChannelInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        return InteractionChannelInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi.V1.InteractionChannelPage>'


class InteractionChannelContext(InstanceContext):

    def __init__(self, version, interaction_sid, sid):
        """
        Initialize the InteractionChannelContext

        :param Version version: Version that contains the resource
        :param interaction_sid: The unique string that identifies the resource
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        """
        super(InteractionChannelContext, self).__init__(version)

        # Path Solution
        self._solution = {'interaction_sid': interaction_sid, 'sid': sid, }
        self._uri = '/Interactions/{interaction_sid}/Channels/{sid}'.format(**self._solution)

        # Dependents
        self._invites = None
        self._participants = None

    def fetch(self):
        """
        Fetch the InteractionChannelInstance

        :returns: The fetched InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return InteractionChannelInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
            sid=self._solution['sid'],
        )

    def update(self, status, routing=values.unset):
        """
        Update the InteractionChannelInstance

        :param InteractionChannelInstance.Status status: Required. The Interaction channels's status
        :param dict routing: Optional. The state of associated tasks.

        :returns: The updated InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        data = values.of({'Status': status, 'Routing': serialize.object(routing), })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return InteractionChannelInstance(
            self._version,
            payload,
            interaction_sid=self._solution['interaction_sid'],
            sid=self._solution['sid'],
        )

    @property
    def invites(self):
        """
        Access the invites

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_invite.InteractionChannelInviteList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_invite.InteractionChannelInviteList
        """
        if self._invites is None:
            self._invites = InteractionChannelInviteList(
                self._version,
                interaction_sid=self._solution['interaction_sid'],
                channel_sid=self._solution['sid'],
            )
        return self._invites

    @property
    def participants(self):
        """
        Access the participants

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        """
        if self._participants is None:
            self._participants = InteractionChannelParticipantList(
                self._version,
                interaction_sid=self._solution['interaction_sid'],
                channel_sid=self._solution['sid'],
            )
        return self._participants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionChannelContext {}>'.format(context)


class InteractionChannelInstance(InstanceResource):

    class Status(object):
        CLOSED = "closed"
        WRAPUP = "wrapup"

    class Type(object):
        VOICE = "voice"
        SMS = "sms"
        EMAIL = "email"
        WEB = "web"
        WHATSAPP = "whatsapp"
        CHAT = "chat"
        MESSENGER = "messenger"
        GBM = "gbm"

    class ChannelStatus(object):
        SETUP = "setup"
        ACTIVE = "active"
        FAILED = "failed"
        CLOSED = "closed"

    def __init__(self, version, payload, interaction_sid, sid=None):
        """
        Initialize the InteractionChannelInstance

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        super(InteractionChannelInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'interaction_sid': payload.get('interaction_sid'),
            'type': payload.get('type'),
            'status': payload.get('status'),
            'error_code': deserialize.integer(payload.get('error_code')),
            'error_message': payload.get('error_message'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'interaction_sid': interaction_sid, 'sid': sid or self._properties['sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: InteractionChannelContext for this InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelContext
        """
        if self._context is None:
            self._context = InteractionChannelContext(
                self._version,
                interaction_sid=self._solution['interaction_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def interaction_sid(self):
        """
        :returns: The unique string that identifies the resource.
        :rtype: unicode
        """
        return self._properties['interaction_sid']

    @property
    def type(self):
        """
        :returns: The Interaction Channel's type.
        :rtype: InteractionChannelInstance.Type
        """
        return self._properties['type']

    @property
    def status(self):
        """
        :returns: The status of this channel.
        :rtype: InteractionChannelInstance.ChannelStatus
        """
        return self._properties['status']

    @property
    def error_code(self):
        """
        :returns: The Twilio error code for a failed channel.
        :rtype: unicode
        """
        return self._properties['error_code']

    @property
    def error_message(self):
        """
        :returns: The error message for a failed channel.
        :rtype: unicode
        """
        return self._properties['error_message']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: The links
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the InteractionChannelInstance

        :returns: The fetched InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        return self._proxy.fetch()

    def update(self, status, routing=values.unset):
        """
        Update the InteractionChannelInstance

        :param InteractionChannelInstance.Status status: Required. The Interaction channels's status
        :param dict routing: Optional. The state of associated tasks.

        :returns: The updated InteractionChannelInstance
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.InteractionChannelInstance
        """
        return self._proxy.update(status, routing=routing, )

    @property
    def invites(self):
        """
        Access the invites

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_invite.InteractionChannelInviteList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_invite.InteractionChannelInviteList
        """
        return self._proxy.invites

    @property
    def participants(self):
        """
        Access the participants

        :returns: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        :rtype: twilio.rest.flex_api.v1.interaction.interaction_channel.interaction_channel_participant.InteractionChannelParticipantList
        """
        return self._proxy.participants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.FlexApi.V1.InteractionChannelInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.preview.understand.assistant import AssistantList


class Understand(Version):

    def __init__(self, domain):
        """
        Initialize the Understand version of Preview

        :returns: Understand version of Preview
        :rtype: twilio.rest.preview.understand.Understand.Understand
        """
        super(Understand, self).__init__(domain)
        self.version = 'understand'
        self._assistants = None

    @property
    def assistants(self):
        """
        :rtype: twilio.rest.preview.understand.assistant.AssistantList
        """
        if self._assistants is None:
            self._assistants = AssistantList(self)
        return self._assistants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.Understand>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page
from twilio.rest.insights.v1.conference.conference_participant import ConferenceParticipantList


class ConferenceList(ListResource):

    def __init__(self, version):
        """
        Initialize the ConferenceList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.insights.v1.conference.ConferenceList
        :rtype: twilio.rest.insights.v1.conference.ConferenceList
        """
        super(ConferenceList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/Conferences'.format(**self._solution)

    def stream(self, conference_sid=values.unset, friendly_name=values.unset,
               status=values.unset, created_after=values.unset,
               created_before=values.unset, mixer_region=values.unset,
               tags=values.unset, subaccount=values.unset,
               detected_issues=values.unset, end_reason=values.unset, limit=None,
               page_size=None):
        """
        Streams ConferenceInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode conference_sid: The SID of the conference.
        :param unicode friendly_name: Custom label for the conference.
        :param unicode status: Conference status.
        :param unicode created_after: Conferences created after timestamp.
        :param unicode created_before: Conferences created before timestamp.
        :param unicode mixer_region: Region where the conference was mixed.
        :param unicode tags: Tags applied by Twilio for common issues.
        :param unicode subaccount: Account SID for the subaccount.
        :param unicode detected_issues: Potential issues detected during the conference.
        :param unicode end_reason: Conference end reason.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.conference.ConferenceInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(
            conference_sid=conference_sid,
            friendly_name=friendly_name,
            status=status,
            created_after=created_after,
            created_before=created_before,
            mixer_region=mixer_region,
            tags=tags,
            subaccount=subaccount,
            detected_issues=detected_issues,
            end_reason=end_reason,
            page_size=limits['page_size'],
        )

        return self._version.stream(page, limits['limit'])

    def list(self, conference_sid=values.unset, friendly_name=values.unset,
             status=values.unset, created_after=values.unset,
             created_before=values.unset, mixer_region=values.unset,
             tags=values.unset, subaccount=values.unset,
             detected_issues=values.unset, end_reason=values.unset, limit=None,
             page_size=None):
        """
        Lists ConferenceInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode conference_sid: The SID of the conference.
        :param unicode friendly_name: Custom label for the conference.
        :param unicode status: Conference status.
        :param unicode created_after: Conferences created after timestamp.
        :param unicode created_before: Conferences created before timestamp.
        :param unicode mixer_region: Region where the conference was mixed.
        :param unicode tags: Tags applied by Twilio for common issues.
        :param unicode subaccount: Account SID for the subaccount.
        :param unicode detected_issues: Potential issues detected during the conference.
        :param unicode end_reason: Conference end reason.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.insights.v1.conference.ConferenceInstance]
        """
        return list(self.stream(
            conference_sid=conference_sid,
            friendly_name=friendly_name,
            status=status,
            created_after=created_after,
            created_before=created_before,
            mixer_region=mixer_region,
            tags=tags,
            subaccount=subaccount,
            detected_issues=detected_issues,
            end_reason=end_reason,
            limit=limit,
            page_size=page_size,
        ))

    def page(self, conference_sid=values.unset, friendly_name=values.unset,
             status=values.unset, created_after=values.unset,
             created_before=values.unset, mixer_region=values.unset,
             tags=values.unset, subaccount=values.unset,
             detected_issues=values.unset, end_reason=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of ConferenceInstance records from the API.
        Request is executed immediately

        :param unicode conference_sid: The SID of the conference.
        :param unicode friendly_name: Custom label for the conference.
        :param unicode status: Conference status.
        :param unicode created_after: Conferences created after timestamp.
        :param unicode created_before: Conferences created before timestamp.
        :param unicode mixer_region: Region where the conference was mixed.
        :param unicode tags: Tags applied by Twilio for common issues.
        :param unicode subaccount: Account SID for the subaccount.
        :param unicode detected_issues: Potential issues detected during the conference.
        :param unicode end_reason: Conference end reason.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferencePage
        """
        data = values.of({
            'ConferenceSid': conference_sid,
            'FriendlyName': friendly_name,
            'Status': status,
            'CreatedAfter': created_after,
            'CreatedBefore': created_before,
            'MixerRegion': mixer_region,
            'Tags': tags,
            'Subaccount': subaccount,
            'DetectedIssues': detected_issues,
            'EndReason': end_reason,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return ConferencePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of ConferenceInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferencePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return ConferencePage(self._version, response, self._solution)

    def get(self, conference_sid):
        """
        Constructs a ConferenceContext

        :param conference_sid: Conference SID.

        :returns: twilio.rest.insights.v1.conference.ConferenceContext
        :rtype: twilio.rest.insights.v1.conference.ConferenceContext
        """
        return ConferenceContext(self._version, conference_sid=conference_sid, )

    def __call__(self, conference_sid):
        """
        Constructs a ConferenceContext

        :param conference_sid: Conference SID.

        :returns: twilio.rest.insights.v1.conference.ConferenceContext
        :rtype: twilio.rest.insights.v1.conference.ConferenceContext
        """
        return ConferenceContext(self._version, conference_sid=conference_sid, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.ConferenceList>'


class ConferencePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ConferencePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.insights.v1.conference.ConferencePage
        :rtype: twilio.rest.insights.v1.conference.ConferencePage
        """
        super(ConferencePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ConferenceInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.insights.v1.conference.ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferenceInstance
        """
        return ConferenceInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1.ConferencePage>'


class ConferenceContext(InstanceContext):

    def __init__(self, version, conference_sid):
        """
        Initialize the ConferenceContext

        :param Version version: Version that contains the resource
        :param conference_sid: Conference SID.

        :returns: twilio.rest.insights.v1.conference.ConferenceContext
        :rtype: twilio.rest.insights.v1.conference.ConferenceContext
        """
        super(ConferenceContext, self).__init__(version)

        # Path Solution
        self._solution = {'conference_sid': conference_sid, }
        self._uri = '/Conferences/{conference_sid}'.format(**self._solution)

        # Dependents
        self._conference_participants = None

    def fetch(self):
        """
        Fetch the ConferenceInstance

        :returns: The fetched ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferenceInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return ConferenceInstance(self._version, payload, conference_sid=self._solution['conference_sid'], )

    @property
    def conference_participants(self):
        """
        Access the conference_participants

        :returns: twilio.rest.insights.v1.conference.conference_participant.ConferenceParticipantList
        :rtype: twilio.rest.insights.v1.conference.conference_participant.ConferenceParticipantList
        """
        if self._conference_participants is None:
            self._conference_participants = ConferenceParticipantList(
                self._version,
                conference_sid=self._solution['conference_sid'],
            )
        return self._conference_participants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.ConferenceContext {}>'.format(context)


class ConferenceInstance(InstanceResource):

    class ConferenceStatus(object):
        IN_PROGRESS = "in_progress"
        NOT_STARTED = "not_started"
        COMPLETED = "completed"
        SUMMARY_TIMEOUT = "summary_timeout"

    class ConferenceEndReason(object):
        LAST_PARTICIPANT_LEFT = "last_participant_left"
        CONFERENCE_ENDED_VIA_API = "conference_ended_via_api"
        PARTICIPANT_WITH_END_CONFERENCE_ON_EXIT_LEFT = "participant_with_end_conference_on_exit_left"
        LAST_PARTICIPANT_KICKED = "last_participant_kicked"
        PARTICIPANT_WITH_END_CONFERENCE_ON_EXIT_KICKED = "participant_with_end_conference_on_exit_kicked"

    class Region(object):
        US1 = "us1"
        AU1 = "au1"
        BR1 = "br1"
        IE1 = "ie1"
        JP1 = "jp1"
        SG1 = "sg1"
        DE1 = "de1"

    class Tag(object):
        INVALID_REQUESTED_REGION = "invalid_requested_region"
        DUPLICATE_IDENTITY = "duplicate_identity"
        START_FAILURE = "start_failure"
        REGION_CONFIGURATION_ISSUES = "region_configuration_issues"
        QUALITY_WARNINGS = "quality_warnings"
        PARTICIPANT_BEHAVIOR_ISSUES = "participant_behavior_issues"
        HIGH_PACKET_LOSS = "high_packet_loss"
        HIGH_JITTER = "high_jitter"
        HIGH_LATENCY = "high_latency"
        LOW_MOS = "low_mos"
        DETECTED_SILENCE = "detected_silence"

    class ProcessingState(object):
        COMPLETE = "complete"
        IN_PROGRESS = "in_progress"
        TIMEOUT = "timeout"

    def __init__(self, version, payload, conference_sid=None):
        """
        Initialize the ConferenceInstance

        :returns: twilio.rest.insights.v1.conference.ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferenceInstance
        """
        super(ConferenceInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'conference_sid': payload.get('conference_sid'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'create_time': deserialize.iso8601_datetime(payload.get('create_time')),
            'start_time': deserialize.iso8601_datetime(payload.get('start_time')),
            'end_time': deserialize.iso8601_datetime(payload.get('end_time')),
            'duration_seconds': deserialize.integer(payload.get('duration_seconds')),
            'connect_duration_seconds': deserialize.integer(payload.get('connect_duration_seconds')),
            'status': payload.get('status'),
            'max_participants': deserialize.integer(payload.get('max_participants')),
            'max_concurrent_participants': deserialize.integer(payload.get('max_concurrent_participants')),
            'unique_participants': deserialize.integer(payload.get('unique_participants')),
            'end_reason': payload.get('end_reason'),
            'ended_by': payload.get('ended_by'),
            'mixer_region': payload.get('mixer_region'),
            'mixer_region_requested': payload.get('mixer_region_requested'),
            'recording_enabled': payload.get('recording_enabled'),
            'detected_issues': payload.get('detected_issues'),
            'tags': payload.get('tags'),
            'tag_info': payload.get('tag_info'),
            'processing_state': payload.get('processing_state'),
            'url': payload.get('url'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'conference_sid': conference_sid or self._properties['conference_sid'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: ConferenceContext for this ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferenceContext
        """
        if self._context is None:
            self._context = ConferenceContext(self._version, conference_sid=self._solution['conference_sid'], )
        return self._context

    @property
    def conference_sid(self):
        """
        :returns: Conference SID.
        :rtype: unicode
        """
        return self._properties['conference_sid']

    @property
    def account_sid(self):
        """
        :returns: Account SID.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: Custom label for the conference.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def create_time(self):
        """
        :returns: Conference creation date/time.
        :rtype: datetime
        """
        return self._properties['create_time']

    @property
    def start_time(self):
        """
        :returns: Timestamp in ISO 8601 format when the conference started.
        :rtype: datetime
        """
        return self._properties['start_time']

    @property
    def end_time(self):
        """
        :returns: Conference end date/time.
        :rtype: datetime
        """
        return self._properties['end_time']

    @property
    def duration_seconds(self):
        """
        :returns: Conference duration in seconds.
        :rtype: unicode
        """
        return self._properties['duration_seconds']

    @property
    def connect_duration_seconds(self):
        """
        :returns: Duration of the conference in seconds.
        :rtype: unicode
        """
        return self._properties['connect_duration_seconds']

    @property
    def status(self):
        """
        :returns: Status of conference
        :rtype: ConferenceInstance.ConferenceStatus
        """
        return self._properties['status']

    @property
    def max_participants(self):
        """
        :returns: Max participants specified in config.
        :rtype: unicode
        """
        return self._properties['max_participants']

    @property
    def max_concurrent_participants(self):
        """
        :returns: Actual maximum concurrent participants.
        :rtype: unicode
        """
        return self._properties['max_concurrent_participants']

    @property
    def unique_participants(self):
        """
        :returns: Unique conference participants.
        :rtype: unicode
        """
        return self._properties['unique_participants']

    @property
    def end_reason(self):
        """
        :returns: Conference end reason.
        :rtype: ConferenceInstance.ConferenceEndReason
        """
        return self._properties['end_reason']

    @property
    def ended_by(self):
        """
        :returns: Call SID that ended the conference.
        :rtype: unicode
        """
        return self._properties['ended_by']

    @property
    def mixer_region(self):
        """
        :returns: Region where the conference was mixed.
        :rtype: ConferenceInstance.Region
        """
        return self._properties['mixer_region']

    @property
    def mixer_region_requested(self):
        """
        :returns: Configuration-requested conference mixer region.
        :rtype: ConferenceInstance.Region
        """
        return self._properties['mixer_region_requested']

    @property
    def recording_enabled(self):
        """
        :returns: Boolean. Indicates whether recording was enabled.
        :rtype: bool
        """
        return self._properties['recording_enabled']

    @property
    def detected_issues(self):
        """
        :returns: Potential issues detected during the conference.
        :rtype: dict
        """
        return self._properties['detected_issues']

    @property
    def tags(self):
        """
        :returns: Tags for detected conference conditions and participant behaviors.
        :rtype: list[ConferenceInstance.Tag]
        """
        return self._properties['tags']

    @property
    def tag_info(self):
        """
        :returns: Object. Contains details about conference tags.
        :rtype: dict
        """
        return self._properties['tag_info']

    @property
    def processing_state(self):
        """
        :returns: Processing state for the Conference Summary resource.
        :rtype: ConferenceInstance.ProcessingState
        """
        return self._properties['processing_state']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def links(self):
        """
        :returns: Nested resource URLs.
        :rtype: unicode
        """
        return self._properties['links']

    def fetch(self):
        """
        Fetch the ConferenceInstance

        :returns: The fetched ConferenceInstance
        :rtype: twilio.rest.insights.v1.conference.ConferenceInstance
        """
        return self._proxy.fetch()

    @property
    def conference_participants(self):
        """
        Access the conference_participants

        :returns: twilio.rest.insights.v1.conference.conference_participant.ConferenceParticipantList
        :rtype: twilio.rest.insights.v1.conference.conference_participant.ConferenceParticipantList
        """
        return self._proxy.conference_participants

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Insights.V1.ConferenceInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.verify.v2 import V2


class Verify(Domain):

    def __init__(self, twilio):
        """
        Initialize the Verify Domain

        :returns: Domain for Verify
        :rtype: twilio.rest.verify.Verify
        """
        super(Verify, self).__init__(twilio)

        self.base_url = 'https://verify.twilio.com'

        # Versions
        self._v2 = None

    @property
    def v2(self):
        """
        :returns: Version v2 of verify
        :rtype: twilio.rest.verify.v2.V2
        """
        if self._v2 is None:
            self._v2 = V2(self)
        return self._v2

    @property
    def forms(self):
        """
        :rtype: twilio.rest.verify.v2.form.FormList
        """
        return self.v2.forms

    @property
    def safelist(self):
        """
        :rtype: twilio.rest.verify.v2.safelist.SafelistList
        """
        return self.v2.safelist

    @property
    def services(self):
        """
        :rtype: twilio.rest.verify.v2.service.ServiceList
        """
        return self.v2.services

    @property
    def verification_attempts(self):
        """
        :rtype: twilio.rest.verify.v2.verification_attempt.VerificationAttemptList
        """
        return self.v2.verification_attempts

    @property
    def verification_attempts_summary(self):
        """
        :rtype: twilio.rest.verify.v2.verification_attempts_summary.VerificationAttemptsSummaryList
        """
        return self.v2.verification_attempts_summary

    @property
    def templates(self):
        """
        :rtype: twilio.rest.verify.v2.template.TemplateList
        """
        return self.v2.templates

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.domain import Domain
from twilio.rest.flex_api.v1 import V1


class FlexApi(Domain):

    def __init__(self, twilio):
        """
        Initialize the FlexApi Domain

        :returns: Domain for FlexApi
        :rtype: twilio.rest.flex_api.FlexApi
        """
        super(FlexApi, self).__init__(twilio)

        self.base_url = 'https://flex-api.twilio.com'

        # Versions
        self._v1 = None

    @property
    def v1(self):
        """
        :returns: Version v1 of flex_api
        :rtype: twilio.rest.flex_api.v1.V1
        """
        if self._v1 is None:
            self._v1 = V1(self)
        return self._v1

    @property
    def channel(self):
        """
        :rtype: twilio.rest.flex_api.v1.channel.ChannelList
        """
        return self.v1.channel

    @property
    def configuration(self):
        """
        :rtype: twilio.rest.flex_api.v1.configuration.ConfigurationList
        """
        return self.v1.configuration

    @property
    def flex_flow(self):
        """
        :rtype: twilio.rest.flex_api.v1.flex_flow.FlexFlowList
        """
        return self.v1.flex_flow

    @property
    def good_data(self):
        """
        :rtype: twilio.rest.flex_api.v1.good_data.GoodDataList
        """
        return self.v1.good_data

    @property
    def interaction(self):
        """
        :rtype: twilio.rest.flex_api.v1.interaction.InteractionList
        """
        return self.v1.interaction

    @property
    def user_roles(self):
        """
        :rtype: twilio.rest.flex_api.v1.user_roles.UserRolesList
        """
        return self.v1.user_roles

    @property
    def web_channel(self):
        """
        :rtype: twilio.rest.flex_api.v1.web_channel.WebChannelList
        """
        return self.v1.web_channel

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.FlexApi>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class TrunkList(ListResource):

    def __init__(self, version):
        """
        Initialize the TrunkList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.routes.v2.trunk.TrunkList
        :rtype: twilio.rest.routes.v2.trunk.TrunkList
        """
        super(TrunkList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, sip_trunk_domain):
        """
        Constructs a TrunkContext

        :param sip_trunk_domain: The SIP Trunk

        :returns: twilio.rest.routes.v2.trunk.TrunkContext
        :rtype: twilio.rest.routes.v2.trunk.TrunkContext
        """
        return TrunkContext(self._version, sip_trunk_domain=sip_trunk_domain, )

    def __call__(self, sip_trunk_domain):
        """
        Constructs a TrunkContext

        :param sip_trunk_domain: The SIP Trunk

        :returns: twilio.rest.routes.v2.trunk.TrunkContext
        :rtype: twilio.rest.routes.v2.trunk.TrunkContext
        """
        return TrunkContext(self._version, sip_trunk_domain=sip_trunk_domain, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes.V2.TrunkList>'


class TrunkPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the TrunkPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.routes.v2.trunk.TrunkPage
        :rtype: twilio.rest.routes.v2.trunk.TrunkPage
        """
        super(TrunkPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of TrunkInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.routes.v2.trunk.TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        return TrunkInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes.V2.TrunkPage>'


class TrunkContext(InstanceContext):

    def __init__(self, version, sip_trunk_domain):
        """
        Initialize the TrunkContext

        :param Version version: Version that contains the resource
        :param sip_trunk_domain: The SIP Trunk

        :returns: twilio.rest.routes.v2.trunk.TrunkContext
        :rtype: twilio.rest.routes.v2.trunk.TrunkContext
        """
        super(TrunkContext, self).__init__(version)

        # Path Solution
        self._solution = {'sip_trunk_domain': sip_trunk_domain, }
        self._uri = '/Trunks/{sip_trunk_domain}'.format(**self._solution)

    def update(self, voice_region=values.unset, friendly_name=values.unset):
        """
        Update the TrunkInstance

        :param unicode voice_region: The Inbound Processing Region used for this SIP Trunk for voice
        :param unicode friendly_name: A human readable description of this resource.

        :returns: The updated TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        data = values.of({'VoiceRegion': voice_region, 'FriendlyName': friendly_name, })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return TrunkInstance(self._version, payload, sip_trunk_domain=self._solution['sip_trunk_domain'], )

    def fetch(self):
        """
        Fetch the TrunkInstance

        :returns: The fetched TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return TrunkInstance(self._version, payload, sip_trunk_domain=self._solution['sip_trunk_domain'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Routes.V2.TrunkContext {}>'.format(context)


class TrunkInstance(InstanceResource):

    def __init__(self, version, payload, sip_trunk_domain=None):
        """
        Initialize the TrunkInstance

        :returns: twilio.rest.routes.v2.trunk.TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        super(TrunkInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sip_trunk_domain': payload.get('sip_trunk_domain'),
            'url': payload.get('url'),
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'friendly_name': payload.get('friendly_name'),
            'voice_region': payload.get('voice_region'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
        }

        # Context
        self._context = None
        self._solution = {'sip_trunk_domain': sip_trunk_domain or self._properties['sip_trunk_domain'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: TrunkContext for this TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkContext
        """
        if self._context is None:
            self._context = TrunkContext(self._version, sip_trunk_domain=self._solution['sip_trunk_domain'], )
        return self._context

    @property
    def sip_trunk_domain(self):
        """
        :returns: The SIP Trunk
        :rtype: unicode
        """
        return self._properties['sip_trunk_domain']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies the Inbound Processing Region assignments for this SIP Trunk.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: Account Sid.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def friendly_name(self):
        """
        :returns: A human readable description of the Inbound Processing Region assignments for this SIP Trunk.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def voice_region(self):
        """
        :returns: The Inbound Processing Region used for this SIP Trunk for voice.
        :rtype: unicode
        """
        return self._properties['voice_region']

    @property
    def date_created(self):
        """
        :returns: The date that this SIP Trunk was assigned an Inbound Processing Region.
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date that the Inbound Processing Region was updated for this SIP Trunk.
        :rtype: datetime
        """
        return self._properties['date_updated']

    def update(self, voice_region=values.unset, friendly_name=values.unset):
        """
        Update the TrunkInstance

        :param unicode voice_region: The Inbound Processing Region used for this SIP Trunk for voice
        :param unicode friendly_name: A human readable description of this resource.

        :returns: The updated TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        return self._proxy.update(voice_region=voice_region, friendly_name=friendly_name, )

    def fetch(self):
        """
        Fetch the TrunkInstance

        :returns: The fetched TrunkInstance
        :rtype: twilio.rest.routes.v2.trunk.TrunkInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Routes.V2.TrunkInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class BrandsInformationList(ListResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the BrandsInformationList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationList
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationList
        """
        super(BrandsInformationList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a BrandsInformationContext

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        """
        return BrandsInformationContext(self._version, )

    def __call__(self):
        """
        Constructs a BrandsInformationContext

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        """
        return BrandsInformationContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.TrustedComms.BrandsInformationList>'


class BrandsInformationPage(Page):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, response, solution):
        """
        Initialize the BrandsInformationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationPage
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationPage
        """
        super(BrandsInformationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of BrandsInformationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        """
        return BrandsInformationInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Preview.TrustedComms.BrandsInformationPage>'


class BrandsInformationContext(InstanceContext):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version):
        """
        Initialize the BrandsInformationContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        """
        super(BrandsInformationContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/BrandsInformation'.format(**self._solution)

    def fetch(self, if_none_match=values.unset):
        """
        Fetch the BrandsInformationInstance

        :param unicode if_none_match: Standard `If-None-Match` HTTP header

        :returns: The fetched BrandsInformationInstance
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        """
        headers = values.of({'If-None-Match': if_none_match, })

        payload = self._version.fetch(method='GET', uri=self._uri, headers=headers, )

        return BrandsInformationInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.TrustedComms.BrandsInformationContext {}>'.format(context)


class BrandsInformationInstance(InstanceResource):
    """ PLEASE NOTE that this class contains preview products that are subject
    to change. Use them with caution. If you currently do not have developer
    preview access, please contact help@twilio.com. """

    def __init__(self, version, payload):
        """
        Initialize the BrandsInformationInstance

        :returns: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        """
        super(BrandsInformationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'update_time': deserialize.iso8601_datetime(payload.get('update_time')),
            'file_link': payload.get('file_link'),
            'file_link_ttl_in_seconds': payload.get('file_link_ttl_in_seconds'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: BrandsInformationContext for this BrandsInformationInstance
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationContext
        """
        if self._context is None:
            self._context = BrandsInformationContext(self._version, )
        return self._context

    @property
    def update_time(self):
        """
        :returns: Creation time of the information retrieved
        :rtype: datetime
        """
        return self._properties['update_time']

    @property
    def file_link(self):
        """
        :returns: The URL to the brands information
        :rtype: unicode
        """
        return self._properties['file_link']

    @property
    def file_link_ttl_in_seconds(self):
        """
        :returns: How long will be the `file_link` valid
        :rtype: unicode
        """
        return self._properties['file_link_ttl_in_seconds']

    @property
    def url(self):
        """
        :returns: The URL of this resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self, if_none_match=values.unset):
        """
        Fetch the BrandsInformationInstance

        :param unicode if_none_match: Standard `If-None-Match` HTTP header

        :returns: The fetched BrandsInformationInstance
        :rtype: twilio.rest.preview.trusted_comms.brands_information.BrandsInformationInstance
        """
        return self._proxy.fetch(if_none_match=if_none_match, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Preview.TrustedComms.BrandsInformationInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class ParticipantConversationList(ListResource):

    def __init__(self, version, chat_service_sid):
        """
        Initialize the ParticipantConversationList

        :param Version version: Version that contains the resource
        :param chat_service_sid: The unique ID of the Conversation Service this conversation belongs to.

        :returns: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationList
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationList
        """
        super(ParticipantConversationList, self).__init__(version)

        # Path Solution
        self._solution = {'chat_service_sid': chat_service_sid, }
        self._uri = '/Services/{chat_service_sid}/ParticipantConversations'.format(**self._solution)

    def stream(self, identity=values.unset, address=values.unset, limit=None,
               page_size=None):
        """
        Streams ParticipantConversationInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode identity: A unique string identifier for the conversation participant as Conversation User.
        :param unicode address: A unique string identifier for the conversation participant who's not a Conversation User.
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(identity=identity, address=address, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, identity=values.unset, address=values.unset, limit=None,
             page_size=None):
        """
        Lists ParticipantConversationInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode identity: A unique string identifier for the conversation participant as Conversation User.
        :param unicode address: A unique string identifier for the conversation participant who's not a Conversation User.
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance]
        """
        return list(self.stream(identity=identity, address=address, limit=limit, page_size=page_size, ))

    def page(self, identity=values.unset, address=values.unset,
             page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of ParticipantConversationInstance records from the API.
        Request is executed immediately

        :param unicode identity: A unique string identifier for the conversation participant as Conversation User.
        :param unicode address: A unique string identifier for the conversation participant who's not a Conversation User.
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of ParticipantConversationInstance
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationPage
        """
        data = values.of({
            'Identity': identity,
            'Address': address,
            'PageToken': page_token,
            'Page': page_number,
            'PageSize': page_size,
        })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return ParticipantConversationPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of ParticipantConversationInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of ParticipantConversationInstance
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return ParticipantConversationPage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.ParticipantConversationList>'


class ParticipantConversationPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the ParticipantConversationPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param chat_service_sid: The unique ID of the Conversation Service this conversation belongs to.

        :returns: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationPage
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationPage
        """
        super(ParticipantConversationPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of ParticipantConversationInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance
        """
        return ParticipantConversationInstance(
            self._version,
            payload,
            chat_service_sid=self._solution['chat_service_sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.ParticipantConversationPage>'


class ParticipantConversationInstance(InstanceResource):

    class State(object):
        INACTIVE = "inactive"
        ACTIVE = "active"
        CLOSED = "closed"

    def __init__(self, version, payload, chat_service_sid):
        """
        Initialize the ParticipantConversationInstance

        :returns: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance
        :rtype: twilio.rest.conversations.v1.service.participant_conversation.ParticipantConversationInstance
        """
        super(ParticipantConversationInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'account_sid': payload.get('account_sid'),
            'chat_service_sid': payload.get('chat_service_sid'),
            'participant_sid': payload.get('participant_sid'),
            'participant_user_sid': payload.get('participant_user_sid'),
            'participant_identity': payload.get('participant_identity'),
            'participant_messaging_binding': payload.get('participant_messaging_binding'),
            'conversation_sid': payload.get('conversation_sid'),
            'conversation_unique_name': payload.get('conversation_unique_name'),
            'conversation_friendly_name': payload.get('conversation_friendly_name'),
            'conversation_attributes': payload.get('conversation_attributes'),
            'conversation_date_created': deserialize.iso8601_datetime(payload.get('conversation_date_created')),
            'conversation_date_updated': deserialize.iso8601_datetime(payload.get('conversation_date_updated')),
            'conversation_created_by': payload.get('conversation_created_by'),
            'conversation_state': payload.get('conversation_state'),
            'conversation_timers': payload.get('conversation_timers'),
            'links': payload.get('links'),
        }

        # Context
        self._context = None
        self._solution = {'chat_service_sid': chat_service_sid, }

    @property
    def account_sid(self):
        """
        :returns: The unique ID of the Account responsible for this conversation.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def chat_service_sid(self):
        """
        :returns: The unique ID of the Conversation Service this conversation belongs to.
        :rtype: unicode
        """
        return self._properties['chat_service_sid']

    @property
    def participant_sid(self):
        """
        :returns: The unique ID of the Participant.
        :rtype: unicode
        """
        return self._properties['participant_sid']

    @property
    def participant_user_sid(self):
        """
        :returns: The unique ID for the conversation participant as Conversation User.
        :rtype: unicode
        """
        return self._properties['participant_user_sid']

    @property
    def participant_identity(self):
        """
        :returns: A unique string identifier for the conversation participant as Conversation User.
        :rtype: unicode
        """
        return self._properties['participant_identity']

    @property
    def participant_messaging_binding(self):
        """
        :returns: Information about how this participant exchanges messages with the conversation.
        :rtype: dict
        """
        return self._properties['participant_messaging_binding']

    @property
    def conversation_sid(self):
        """
        :returns: The unique ID of the Conversation this Participant belongs to.
        :rtype: unicode
        """
        return self._properties['conversation_sid']

    @property
    def conversation_unique_name(self):
        """
        :returns: An application-defined string that uniquely identifies the Conversation resource.
        :rtype: unicode
        """
        return self._properties['conversation_unique_name']

    @property
    def conversation_friendly_name(self):
        """
        :returns: The human-readable name of this conversation.
        :rtype: unicode
        """
        return self._properties['conversation_friendly_name']

    @property
    def conversation_attributes(self):
        """
        :returns: An optional string metadata field you can use to store any data you wish.
        :rtype: unicode
        """
        return self._properties['conversation_attributes']

    @property
    def conversation_date_created(self):
        """
        :returns: The date that this conversation was created.
        :rtype: datetime
        """
        return self._properties['conversation_date_created']

    @property
    def conversation_date_updated(self):
        """
        :returns: The date that this conversation was last updated.
        :rtype: datetime
        """
        return self._properties['conversation_date_updated']

    @property
    def conversation_created_by(self):
        """
        :returns: Creator of this conversation.
        :rtype: unicode
        """
        return self._properties['conversation_created_by']

    @property
    def conversation_state(self):
        """
        :returns: The current state of this User Conversation
        :rtype: ParticipantConversationInstance.State
        """
        return self._properties['conversation_state']

    @property
    def conversation_timers(self):
        """
        :returns: Timer date values for this conversation.
        :rtype: dict
        """
        return self._properties['conversation_timers']

    @property
    def links(self):
        """
        :returns: Absolute URLs to access the participant and conversation of this Participant Conversation.
        :rtype: unicode
        """
        return self._properties['links']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Conversations.V1.ParticipantConversationInstance>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.insights.v1.call import CallList
from twilio.rest.insights.v1.call_summaries import CallSummariesList
from twilio.rest.insights.v1.conference import ConferenceList
from twilio.rest.insights.v1.room import RoomList
from twilio.rest.insights.v1.setting import SettingList


class V1(Version):

    def __init__(self, domain):
        """
        Initialize the V1 version of Insights

        :returns: V1 version of Insights
        :rtype: twilio.rest.insights.v1.V1.V1
        """
        super(V1, self).__init__(domain)
        self.version = 'v1'
        self._settings = None
        self._calls = None
        self._call_summaries = None
        self._conferences = None
        self._rooms = None

    @property
    def settings(self):
        """
        :rtype: twilio.rest.insights.v1.setting.SettingList
        """
        if self._settings is None:
            self._settings = SettingList(self)
        return self._settings

    @property
    def calls(self):
        """
        :rtype: twilio.rest.insights.v1.call.CallList
        """
        if self._calls is None:
            self._calls = CallList(self)
        return self._calls

    @property
    def call_summaries(self):
        """
        :rtype: twilio.rest.insights.v1.call_summaries.CallSummariesList
        """
        if self._call_summaries is None:
            self._call_summaries = CallSummariesList(self)
        return self._call_summaries

    @property
    def conferences(self):
        """
        :rtype: twilio.rest.insights.v1.conference.ConferenceList
        """
        if self._conferences is None:
            self._conferences = ConferenceList(self)
        return self._conferences

    @property
    def rooms(self):
        """
        :rtype: twilio.rest.insights.v1.room.RoomList
        """
        if self._rooms is None:
            self._rooms = RoomList(self)
        return self._rooms

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Insights.V1>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class BuildStatusList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, service_sid, sid):
        """
        Initialize the BuildStatusList

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service that the Build resource is associated with
        :param sid: The unique string that identifies the Build resource

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusList
        """
        super(BuildStatusList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'sid': sid, }

    def get(self):
        """
        Constructs a BuildStatusContext

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        """
        return BuildStatusContext(
            self._version,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def __call__(self):
        """
        Constructs a BuildStatusContext

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        """
        return BuildStatusContext(
            self._version,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Serverless.V1.BuildStatusList>'


class BuildStatusPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the BuildStatusPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: The SID of the Service that the Build resource is associated with
        :param sid: The unique string that identifies the Build resource

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusPage
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusPage
        """
        super(BuildStatusPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of BuildStatusInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        """
        return BuildStatusInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Serverless.V1.BuildStatusPage>'


class BuildStatusContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, service_sid, sid):
        """
        Initialize the BuildStatusContext

        :param Version version: Version that contains the resource
        :param service_sid: The SID of the Service to fetch the Build resource from
        :param sid: The SID of the Build resource to fetch

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        """
        super(BuildStatusContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'sid': sid, }
        self._uri = '/Services/{service_sid}/Builds/{sid}/Status'.format(**self._solution)

    def fetch(self):
        """
        Fetch the BuildStatusInstance

        :returns: The fetched BuildStatusInstance
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return BuildStatusInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Serverless.V1.BuildStatusContext {}>'.format(context)


class BuildStatusInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Status(object):
        BUILDING = "building"
        COMPLETED = "completed"
        FAILED = "failed"

    def __init__(self, version, payload, service_sid, sid):
        """
        Initialize the BuildStatusInstance

        :returns: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        """
        super(BuildStatusInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'status': payload.get('status'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'service_sid': service_sid, 'sid': sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: BuildStatusContext for this BuildStatusInstance
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusContext
        """
        if self._context is None:
            self._context = BuildStatusContext(
                self._version,
                service_sid=self._solution['service_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the Build resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the Build resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: The SID of the Service that the Build resource is associated with
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def status(self):
        """
        :returns: The status of the Build
        :rtype: BuildStatusInstance.Status
        """
        return self._properties['status']

    @property
    def url(self):
        """
        :returns: The absolute URL of the Build Status resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the BuildStatusInstance

        :returns: The fetched BuildStatusInstance
        :rtype: twilio.rest.serverless.v1.service.build.build_status.BuildStatusInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Serverless.V1.BuildStatusInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class SafelistList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the SafelistList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.verify.v2.safelist.SafelistList
        :rtype: twilio.rest.verify.v2.safelist.SafelistList
        """
        super(SafelistList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/SafeList/Numbers'.format(**self._solution)

    def create(self, phone_number):
        """
        Create the SafelistInstance

        :param unicode phone_number: The phone number to be added in SafeList.

        :returns: The created SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistInstance
        """
        data = values.of({'PhoneNumber': phone_number, })

        payload = self._version.create(method='POST', uri=self._uri, data=data, )

        return SafelistInstance(self._version, payload, )

    def get(self, phone_number):
        """
        Constructs a SafelistContext

        :param phone_number: The phone number to be fetched from SafeList.

        :returns: twilio.rest.verify.v2.safelist.SafelistContext
        :rtype: twilio.rest.verify.v2.safelist.SafelistContext
        """
        return SafelistContext(self._version, phone_number=phone_number, )

    def __call__(self, phone_number):
        """
        Constructs a SafelistContext

        :param phone_number: The phone number to be fetched from SafeList.

        :returns: twilio.rest.verify.v2.safelist.SafelistContext
        :rtype: twilio.rest.verify.v2.safelist.SafelistContext
        """
        return SafelistContext(self._version, phone_number=phone_number, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.SafelistList>'


class SafelistPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the SafelistPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.verify.v2.safelist.SafelistPage
        :rtype: twilio.rest.verify.v2.safelist.SafelistPage
        """
        super(SafelistPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of SafelistInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.safelist.SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistInstance
        """
        return SafelistInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.SafelistPage>'


class SafelistContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, phone_number):
        """
        Initialize the SafelistContext

        :param Version version: Version that contains the resource
        :param phone_number: The phone number to be fetched from SafeList.

        :returns: twilio.rest.verify.v2.safelist.SafelistContext
        :rtype: twilio.rest.verify.v2.safelist.SafelistContext
        """
        super(SafelistContext, self).__init__(version)

        # Path Solution
        self._solution = {'phone_number': phone_number, }
        self._uri = '/SafeList/Numbers/{phone_number}'.format(**self._solution)

    def fetch(self):
        """
        Fetch the SafelistInstance

        :returns: The fetched SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return SafelistInstance(self._version, payload, phone_number=self._solution['phone_number'], )

    def delete(self):
        """
        Deletes the SafelistInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.SafelistContext {}>'.format(context)


class SafelistInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, payload, phone_number=None):
        """
        Initialize the SafelistInstance

        :returns: twilio.rest.verify.v2.safelist.SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistInstance
        """
        super(SafelistInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'phone_number': payload.get('phone_number'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'phone_number': phone_number or self._properties['phone_number'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: SafelistContext for this SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistContext
        """
        if self._context is None:
            self._context = SafelistContext(self._version, phone_number=self._solution['phone_number'], )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def phone_number(self):
        """
        :returns: The phone number in SafeList.
        :rtype: unicode
        """
        return self._properties['phone_number']

    @property
    def url(self):
        """
        :returns: The absolute URL of the SafeList resource.
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the SafelistInstance

        :returns: The fetched SafelistInstance
        :rtype: twilio.rest.verify.v2.safelist.SafelistInstance
        """
        return self._proxy.fetch()

    def delete(self):
        """
        Deletes the SafelistInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.SafelistInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class OauthList(ListResource):

    def __init__(self, version):
        """
        Initialize the OauthList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.oauth.OauthList
        :rtype: twilio.rest.oauth.v1.oauth.OauthList
        """
        super(OauthList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self):
        """
        Constructs a OauthContext

        :returns: twilio.rest.oauth.v1.oauth.OauthContext
        :rtype: twilio.rest.oauth.v1.oauth.OauthContext
        """
        return OauthContext(self._version, )

    def __call__(self):
        """
        Constructs a OauthContext

        :returns: twilio.rest.oauth.v1.oauth.OauthContext
        :rtype: twilio.rest.oauth.v1.oauth.OauthContext
        """
        return OauthContext(self._version, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.OauthList>'


class OauthPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the OauthPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.oauth.v1.oauth.OauthPage
        :rtype: twilio.rest.oauth.v1.oauth.OauthPage
        """
        super(OauthPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of OauthInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.oauth.v1.oauth.OauthInstance
        :rtype: twilio.rest.oauth.v1.oauth.OauthInstance
        """
        return OauthInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Oauth.V1.OauthPage>'


class OauthContext(InstanceContext):

    def __init__(self, version):
        """
        Initialize the OauthContext

        :param Version version: Version that contains the resource

        :returns: twilio.rest.oauth.v1.oauth.OauthContext
        :rtype: twilio.rest.oauth.v1.oauth.OauthContext
        """
        super(OauthContext, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/certs'.format(**self._solution)

    def fetch(self):
        """
        Fetch the OauthInstance

        :returns: The fetched OauthInstance
        :rtype: twilio.rest.oauth.v1.oauth.OauthInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return OauthInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.OauthContext {}>'.format(context)


class OauthInstance(InstanceResource):

    def __init__(self, version, payload):
        """
        Initialize the OauthInstance

        :returns: twilio.rest.oauth.v1.oauth.OauthInstance
        :rtype: twilio.rest.oauth.v1.oauth.OauthInstance
        """
        super(OauthInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {'keys': payload.get('keys'), 'url': payload.get('url'), }

        # Context
        self._context = None
        self._solution = {}

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: OauthContext for this OauthInstance
        :rtype: twilio.rest.oauth.v1.oauth.OauthContext
        """
        if self._context is None:
            self._context = OauthContext(self._version, )
        return self._context

    @property
    def keys(self):
        """
        :returns: A collection of certificates
        :rtype: dict
        """
        return self._properties['keys']

    @property
    def url(self):
        """
        :returns: The url
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self):
        """
        Fetch the OauthInstance

        :returns: The fetched OauthInstance
        :rtype: twilio.rest.oauth.v1.oauth.OauthInstance
        """
        return self._proxy.fetch()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Oauth.V1.OauthInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class PhoneNumberList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the PhoneNumberList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberList
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberList
        """
        super(PhoneNumberList, self).__init__(version)

        # Path Solution
        self._solution = {}

    def get(self, phone_number):
        """
        Constructs a PhoneNumberContext

        :param phone_number: Phone number to lookup

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        """
        return PhoneNumberContext(self._version, phone_number=phone_number, )

    def __call__(self, phone_number):
        """
        Constructs a PhoneNumberContext

        :param phone_number: Phone number to lookup

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        """
        return PhoneNumberContext(self._version, phone_number=phone_number, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Lookups.V2.PhoneNumberList>'


class PhoneNumberPage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the PhoneNumberPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberPage
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberPage
        """
        super(PhoneNumberPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of PhoneNumberInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        """
        return PhoneNumberInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Lookups.V2.PhoneNumberPage>'


class PhoneNumberContext(InstanceContext):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, phone_number):
        """
        Initialize the PhoneNumberContext

        :param Version version: Version that contains the resource
        :param phone_number: Phone number to lookup

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        """
        super(PhoneNumberContext, self).__init__(version)

        # Path Solution
        self._solution = {'phone_number': phone_number, }
        self._uri = '/PhoneNumbers/{phone_number}'.format(**self._solution)

    def fetch(self, fields=values.unset, country_code=values.unset):
        """
        Fetch the PhoneNumberInstance

        :param unicode fields: Fields to return
        :param unicode country_code: Country code for national phone number lookups

        :returns: The fetched PhoneNumberInstance
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        """
        data = values.of({'Fields': fields, 'CountryCode': country_code, })

        payload = self._version.fetch(method='GET', uri=self._uri, params=data, )

        return PhoneNumberInstance(self._version, payload, phone_number=self._solution['phone_number'], )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Lookups.V2.PhoneNumberContext {}>'.format(context)


class PhoneNumberInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Type(object):
        LANDLINE = "landline"
        MOBILE = "mobile"
        VOIP = "voip"

    class ValidationError(object):
        TOO_SHORT = "TOO_SHORT"
        TOO_LONG = "TOO_LONG"
        INVALID_BUT_POSSIBLE = "INVALID_BUT_POSSIBLE"
        INVALID_COUNTRY_CODE = "INVALID_COUNTRY_CODE"
        INVALID_LENGTH = "INVALID_LENGTH"
        NOT_A_NUMBER = "NOT_A_NUMBER"

    def __init__(self, version, payload, phone_number=None):
        """
        Initialize the PhoneNumberInstance

        :returns: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        """
        super(PhoneNumberInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'calling_country_code': payload.get('calling_country_code'),
            'country_code': payload.get('country_code'),
            'phone_number': payload.get('phone_number'),
            'national_format': payload.get('national_format'),
            'valid': payload.get('valid'),
            'validation_errors': payload.get('validation_errors'),
            'caller_name': payload.get('caller_name'),
            'sim_swap': payload.get('sim_swap'),
            'call_forwarding': payload.get('call_forwarding'),
            'live_activity': payload.get('live_activity'),
            'line_type_intelligence': payload.get('line_type_intelligence'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'phone_number': phone_number or self._properties['phone_number'], }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: PhoneNumberContext for this PhoneNumberInstance
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberContext
        """
        if self._context is None:
            self._context = PhoneNumberContext(self._version, phone_number=self._solution['phone_number'], )
        return self._context

    @property
    def calling_country_code(self):
        """
        :returns: International dialing prefix
        :rtype: unicode
        """
        return self._properties['calling_country_code']

    @property
    def country_code(self):
        """
        :returns: Phone number's ISO country code
        :rtype: unicode
        """
        return self._properties['country_code']

    @property
    def phone_number(self):
        """
        :returns: Phone number in E.164 format
        :rtype: unicode
        """
        return self._properties['phone_number']

    @property
    def national_format(self):
        """
        :returns: Phone number in national format
        :rtype: unicode
        """
        return self._properties['national_format']

    @property
    def valid(self):
        """
        :returns: Boolean which indicates if the phone number is valid
        :rtype: bool
        """
        return self._properties['valid']

    @property
    def validation_errors(self):
        """
        :returns: Contains reasons why a phone number is invalid
        :rtype: list[PhoneNumberInstance.ValidationError]
        """
        return self._properties['validation_errors']

    @property
    def caller_name(self):
        """
        :returns: An object that contains caller name information
        :rtype: dict
        """
        return self._properties['caller_name']

    @property
    def sim_swap(self):
        """
        :returns: An object that contains SIM swap information
        :rtype: dict
        """
        return self._properties['sim_swap']

    @property
    def call_forwarding(self):
        """
        :returns: An object that contains call forwarding status information
        :rtype: dict
        """
        return self._properties['call_forwarding']

    @property
    def live_activity(self):
        """
        :returns: An object that contains live activity information
        :rtype: dict
        """
        return self._properties['live_activity']

    @property
    def line_type_intelligence(self):
        """
        :returns: An object that contains line type information
        :rtype: dict
        """
        return self._properties['line_type_intelligence']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    def fetch(self, fields=values.unset, country_code=values.unset):
        """
        Fetch the PhoneNumberInstance

        :param unicode fields: Fields to return
        :param unicode country_code: Country code for national phone number lookups

        :returns: The fetched PhoneNumberInstance
        :rtype: twilio.rest.lookups.v2.phone_number.PhoneNumberInstance
        """
        return self._proxy.fetch(fields=fields, country_code=country_code, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Lookups.V2.PhoneNumberInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base.version import Version
from twilio.rest.routes.v2.phone_number import PhoneNumberList
from twilio.rest.routes.v2.sip_domain import SipDomainList
from twilio.rest.routes.v2.trunk import TrunkList


class V2(Version):

    def __init__(self, domain):
        """
        Initialize the V2 version of Routes

        :returns: V2 version of Routes
        :rtype: twilio.rest.routes.v2.V2.V2
        """
        super(V2, self).__init__(domain)
        self.version = 'v2'
        self._phone_numbers = None
        self._sip_domains = None
        self._trunks = None

    @property
    def phone_numbers(self):
        """
        :rtype: twilio.rest.routes.v2.phone_number.PhoneNumberList
        """
        if self._phone_numbers is None:
            self._phone_numbers = PhoneNumberList(self)
        return self._phone_numbers

    @property
    def sip_domains(self):
        """
        :rtype: twilio.rest.routes.v2.sip_domain.SipDomainList
        """
        if self._sip_domains is None:
            self._sip_domains = SipDomainList(self)
        return self._sip_domains

    @property
    def trunks(self):
        """
        :rtype: twilio.rest.routes.v2.trunk.TrunkList
        """
        if self._trunks is None:
            self._trunks = TrunkList(self)
        return self._trunks

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Routes.V2>'



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class AnonymizeList(ListResource):

    def __init__(self, version, room_sid, sid):
        """
        Initialize the AnonymizeList

        :param Version version: Version that contains the resource
        :param room_sid: The SID of the participant's room
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeList
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeList
        """
        super(AnonymizeList, self).__init__(version)

        # Path Solution
        self._solution = {'room_sid': room_sid, 'sid': sid, }

    def get(self):
        """
        Constructs a AnonymizeContext

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        """
        return AnonymizeContext(
            self._version,
            room_sid=self._solution['room_sid'],
            sid=self._solution['sid'],
        )

    def __call__(self):
        """
        Constructs a AnonymizeContext

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        """
        return AnonymizeContext(
            self._version,
            room_sid=self._solution['room_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Video.V1.AnonymizeList>'


class AnonymizePage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the AnonymizePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param room_sid: The SID of the participant's room
        :param sid: The unique string that identifies the resource

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizePage
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizePage
        """
        super(AnonymizePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of AnonymizeInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        """
        return AnonymizeInstance(
            self._version,
            payload,
            room_sid=self._solution['room_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Video.V1.AnonymizePage>'


class AnonymizeContext(InstanceContext):

    def __init__(self, version, room_sid, sid):
        """
        Initialize the AnonymizeContext

        :param Version version: Version that contains the resource
        :param room_sid: The SID of the room with the participant to update
        :param sid: The SID that identifies the resource to update

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        """
        super(AnonymizeContext, self).__init__(version)

        # Path Solution
        self._solution = {'room_sid': room_sid, 'sid': sid, }
        self._uri = '/Rooms/{room_sid}/Participants/{sid}/Anonymize'.format(**self._solution)

    def update(self):
        """
        Update the AnonymizeInstance

        :returns: The updated AnonymizeInstance
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        """
        payload = self._version.update(method='POST', uri=self._uri, )

        return AnonymizeInstance(
            self._version,
            payload,
            room_sid=self._solution['room_sid'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Video.V1.AnonymizeContext {}>'.format(context)


class AnonymizeInstance(InstanceResource):

    class Status(object):
        CONNECTED = "connected"
        DISCONNECTED = "disconnected"

    def __init__(self, version, payload, room_sid, sid):
        """
        Initialize the AnonymizeInstance

        :returns: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        """
        super(AnonymizeInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'room_sid': payload.get('room_sid'),
            'account_sid': payload.get('account_sid'),
            'status': payload.get('status'),
            'identity': payload.get('identity'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'start_time': deserialize.iso8601_datetime(payload.get('start_time')),
            'end_time': deserialize.iso8601_datetime(payload.get('end_time')),
            'duration': deserialize.integer(payload.get('duration')),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {'room_sid': room_sid, 'sid': sid, }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: AnonymizeContext for this AnonymizeInstance
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeContext
        """
        if self._context is None:
            self._context = AnonymizeContext(
                self._version,
                room_sid=self._solution['room_sid'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: The unique string that identifies the resource
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def room_sid(self):
        """
        :returns: The SID of the participant's room
        :rtype: unicode
        """
        return self._properties['room_sid']

    @property
    def account_sid(self):
        """
        :returns: The SID of the Account that created the resource
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def status(self):
        """
        :returns: The status of the Participant
        :rtype: AnonymizeInstance.Status
        """
        return self._properties['status']

    @property
    def identity(self):
        """
        :returns: The SID of the participant
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def date_created(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The ISO 8601 date and time in GMT when the resource was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def start_time(self):
        """
        :returns: The time of participant connected to the room in ISO 8601 format
        :rtype: datetime
        """
        return self._properties['start_time']

    @property
    def end_time(self):
        """
        :returns: The time when the participant disconnected from the room in ISO 8601 format
        :rtype: datetime
        """
        return self._properties['end_time']

    @property
    def duration(self):
        """
        :returns: Duration of time in seconds the participant was connected
        :rtype: unicode
        """
        return self._properties['duration']

    @property
    def url(self):
        """
        :returns: The absolute URL of the resource
        :rtype: unicode
        """
        return self._properties['url']

    def update(self):
        """
        Update the AnonymizeInstance

        :returns: The updated AnonymizeInstance
        :rtype: twilio.rest.video.v1.room.room_participant.room_participant_anonymize.AnonymizeInstance
        """
        return self._proxy.update()

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Video.V1.AnonymizeInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_context import InstanceContext
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class FactorList(ListResource):

    def __init__(self, version, service_sid, identity):
        """
        Initialize the FactorList

        :param Version version: Version that contains the resource
        :param service_sid: Service Sid.
        :param identity: Unique external identifier of the Entity

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorList
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorList
        """
        super(FactorList, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'identity': identity, }
        self._uri = '/Services/{service_sid}/Entities/{identity}/Factors'.format(**self._solution)

    def stream(self, limit=None, page_size=None):
        """
        Streams FactorInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.entity.factor.FactorInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, limit=None, page_size=None):
        """
        Lists FactorInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.verify.v2.service.entity.factor.FactorInstance]
        """
        return list(self.stream(limit=limit, page_size=page_size, ))

    def page(self, page_token=values.unset, page_number=values.unset,
             page_size=values.unset):
        """
        Retrieve a single page of FactorInstance records from the API.
        Request is executed immediately

        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorPage
        """
        data = values.of({'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return FactorPage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of FactorInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorPage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return FactorPage(self._version, response, self._solution)

    def get(self, sid):
        """
        Constructs a FactorContext

        :param sid: A string that uniquely identifies this Factor.

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorContext
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorContext
        """
        return FactorContext(
            self._version,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
            sid=sid,
        )

    def __call__(self, sid):
        """
        Constructs a FactorContext

        :param sid: A string that uniquely identifies this Factor.

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorContext
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorContext
        """
        return FactorContext(
            self._version,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
            sid=sid,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.FactorList>'


class FactorPage(Page):

    def __init__(self, version, response, solution):
        """
        Initialize the FactorPage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API
        :param service_sid: Service Sid.
        :param identity: Unique external identifier of the Entity

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorPage
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorPage
        """
        super(FactorPage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of FactorInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        return FactorInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Verify.V2.FactorPage>'


class FactorContext(InstanceContext):

    def __init__(self, version, service_sid, identity, sid):
        """
        Initialize the FactorContext

        :param Version version: Version that contains the resource
        :param service_sid: Service Sid.
        :param identity: Unique external identifier of the Entity
        :param sid: A string that uniquely identifies this Factor.

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorContext
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorContext
        """
        super(FactorContext, self).__init__(version)

        # Path Solution
        self._solution = {'service_sid': service_sid, 'identity': identity, 'sid': sid, }
        self._uri = '/Services/{service_sid}/Entities/{identity}/Factors/{sid}'.format(**self._solution)

    def delete(self):
        """
        Deletes the FactorInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._version.delete(method='DELETE', uri=self._uri, )

    def fetch(self):
        """
        Fetch the FactorInstance

        :returns: The fetched FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        payload = self._version.fetch(method='GET', uri=self._uri, )

        return FactorInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
            sid=self._solution['sid'],
        )

    def update(self, auth_payload=values.unset, friendly_name=values.unset,
               config_notification_token=values.unset,
               config_sdk_version=values.unset, config_time_step=values.unset,
               config_skew=values.unset, config_code_length=values.unset,
               config_alg=values.unset, config_notification_platform=values.unset):
        """
        Update the FactorInstance

        :param unicode auth_payload: Optional payload to verify the Factor for the first time
        :param unicode friendly_name: The friendly name of this Factor
        :param unicode config_notification_token: For APN, the device token. For FCM, the registration token
        :param unicode config_sdk_version: The Verify Push SDK version used to configure the factor
        :param unicode config_time_step: How often, in seconds, are TOTP codes generated
        :param unicode config_skew: The number of past and future time-steps valid at a given time
        :param unicode config_code_length: Number of digits for generated TOTP codes
        :param FactorInstance.TotpAlgorithms config_alg: The algorithm used to derive the TOTP codes
        :param unicode config_notification_platform: The transport technology used to generate the Notification Token

        :returns: The updated FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        data = values.of({
            'AuthPayload': auth_payload,
            'FriendlyName': friendly_name,
            'Config.NotificationToken': config_notification_token,
            'Config.SdkVersion': config_sdk_version,
            'Config.TimeStep': config_time_step,
            'Config.Skew': config_skew,
            'Config.CodeLength': config_code_length,
            'Config.Alg': config_alg,
            'Config.NotificationPlatform': config_notification_platform,
        })

        payload = self._version.update(method='POST', uri=self._uri, data=data, )

        return FactorInstance(
            self._version,
            payload,
            service_sid=self._solution['service_sid'],
            identity=self._solution['identity'],
            sid=self._solution['sid'],
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.FactorContext {}>'.format(context)


class FactorInstance(InstanceResource):

    class FactorStatuses(object):
        UNVERIFIED = "unverified"
        VERIFIED = "verified"

    class FactorTypes(object):
        PUSH = "push"
        TOTP = "totp"

    class NotificationPlatforms(object):
        APN = "apn"
        FCM = "fcm"
        NONE = "none"

    class TotpAlgorithms(object):
        SHA1 = "sha1"
        SHA256 = "sha256"
        SHA512 = "sha512"

    def __init__(self, version, payload, service_sid, identity, sid=None):
        """
        Initialize the FactorInstance

        :returns: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        super(FactorInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'account_sid': payload.get('account_sid'),
            'service_sid': payload.get('service_sid'),
            'entity_sid': payload.get('entity_sid'),
            'identity': payload.get('identity'),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
            'friendly_name': payload.get('friendly_name'),
            'status': payload.get('status'),
            'factor_type': payload.get('factor_type'),
            'config': payload.get('config'),
            'metadata': payload.get('metadata'),
            'url': payload.get('url'),
        }

        # Context
        self._context = None
        self._solution = {
            'service_sid': service_sid,
            'identity': identity,
            'sid': sid or self._properties['sid'],
        }

    @property
    def _proxy(self):
        """
        Generate an instance context for the instance, the context is capable of
        performing various actions.  All instance actions are proxied to the context

        :returns: FactorContext for this FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorContext
        """
        if self._context is None:
            self._context = FactorContext(
                self._version,
                service_sid=self._solution['service_sid'],
                identity=self._solution['identity'],
                sid=self._solution['sid'],
            )
        return self._context

    @property
    def sid(self):
        """
        :returns: A string that uniquely identifies this Factor.
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def account_sid(self):
        """
        :returns: Account Sid.
        :rtype: unicode
        """
        return self._properties['account_sid']

    @property
    def service_sid(self):
        """
        :returns: Service Sid.
        :rtype: unicode
        """
        return self._properties['service_sid']

    @property
    def entity_sid(self):
        """
        :returns: Entity Sid.
        :rtype: unicode
        """
        return self._properties['entity_sid']

    @property
    def identity(self):
        """
        :returns: Unique external identifier of the Entity
        :rtype: unicode
        """
        return self._properties['identity']

    @property
    def date_created(self):
        """
        :returns: The date this Factor was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date this Factor was updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    @property
    def friendly_name(self):
        """
        :returns: A human readable description of this resource.
        :rtype: unicode
        """
        return self._properties['friendly_name']

    @property
    def status(self):
        """
        :returns: The Status of this Factor
        :rtype: FactorInstance.FactorStatuses
        """
        return self._properties['status']

    @property
    def factor_type(self):
        """
        :returns: The Type of this Factor
        :rtype: FactorInstance.FactorTypes
        """
        return self._properties['factor_type']

    @property
    def config(self):
        """
        :returns: Configurations for a `factor_type`.
        :rtype: dict
        """
        return self._properties['config']

    @property
    def metadata(self):
        """
        :returns: Metadata of the factor.
        :rtype: dict
        """
        return self._properties['metadata']

    @property
    def url(self):
        """
        :returns: The URL of this resource.
        :rtype: unicode
        """
        return self._properties['url']

    def delete(self):
        """
        Deletes the FactorInstance

        :returns: True if delete succeeds, False otherwise
        :rtype: bool
        """
        return self._proxy.delete()

    def fetch(self):
        """
        Fetch the FactorInstance

        :returns: The fetched FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        return self._proxy.fetch()

    def update(self, auth_payload=values.unset, friendly_name=values.unset,
               config_notification_token=values.unset,
               config_sdk_version=values.unset, config_time_step=values.unset,
               config_skew=values.unset, config_code_length=values.unset,
               config_alg=values.unset, config_notification_platform=values.unset):
        """
        Update the FactorInstance

        :param unicode auth_payload: Optional payload to verify the Factor for the first time
        :param unicode friendly_name: The friendly name of this Factor
        :param unicode config_notification_token: For APN, the device token. For FCM, the registration token
        :param unicode config_sdk_version: The Verify Push SDK version used to configure the factor
        :param unicode config_time_step: How often, in seconds, are TOTP codes generated
        :param unicode config_skew: The number of past and future time-steps valid at a given time
        :param unicode config_code_length: Number of digits for generated TOTP codes
        :param FactorInstance.TotpAlgorithms config_alg: The algorithm used to derive the TOTP codes
        :param unicode config_notification_platform: The transport technology used to generate the Notification Token

        :returns: The updated FactorInstance
        :rtype: twilio.rest.verify.v2.service.entity.factor.FactorInstance
        """
        return self._proxy.update(
            auth_payload=auth_payload,
            friendly_name=friendly_name,
            config_notification_token=config_notification_token,
            config_sdk_version=config_sdk_version,
            config_time_step=config_time_step,
            config_skew=config_skew,
            config_code_length=config_code_length,
            config_alg=config_alg,
            config_notification_platform=config_notification_platform,
        )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        context = ' '.join('{}={}'.format(k, v) for k, v in self._solution.items())
        return '<Twilio.Verify.V2.FactorInstance {}>'.format(context)



# coding=utf-8
r"""
This code was generated by
\ / _    _  _|   _  _
 | (_)\/(_)(_|\/| |(/_  v1.0.0
      /       /
"""

from twilio.base import deserialize
from twilio.base import values
from twilio.base.instance_resource import InstanceResource
from twilio.base.list_resource import ListResource
from twilio.base.page import Page


class SettingsUpdateList(ListResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version):
        """
        Initialize the SettingsUpdateList

        :param Version version: Version that contains the resource

        :returns: twilio.rest.supersim.v1.settings_update.SettingsUpdateList
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdateList
        """
        super(SettingsUpdateList, self).__init__(version)

        # Path Solution
        self._solution = {}
        self._uri = '/SettingsUpdates'.format(**self._solution)

    def stream(self, sim=values.unset, limit=None, page_size=None):
        """
        Streams SettingsUpdateInstance records from the API as a generator stream.
        This operation lazily loads records as efficiently as possible until the limit
        is reached.
        The results are returned as a generator, so this operation is memory efficient.

        :param unicode sim: Filter the Settings Updates by Super SIM
        :param int limit: Upper limit for the number of records to return. stream()
                          guarantees to never return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, stream() will attempt to read the
                              limit with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance]
        """
        limits = self._version.read_limits(limit, page_size)

        page = self.page(sim=sim, page_size=limits['page_size'], )

        return self._version.stream(page, limits['limit'])

    def list(self, sim=values.unset, limit=None, page_size=None):
        """
        Lists SettingsUpdateInstance records from the API as a list.
        Unlike stream(), this operation is eager and will load `limit` records into
        memory before returning.

        :param unicode sim: Filter the Settings Updates by Super SIM
        :param int limit: Upper limit for the number of records to return. list() guarantees
                          never to return more than limit.  Default is no limit
        :param int page_size: Number of records to fetch per request, when not set will use
                              the default value of 50 records.  If no page_size is defined
                              but a limit is defined, list() will attempt to read the limit
                              with the most efficient page size, i.e. min(limit, 1000)

        :returns: Generator that will yield up to limit results
        :rtype: list[twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance]
        """
        return list(self.stream(sim=sim, limit=limit, page_size=page_size, ))

    def page(self, sim=values.unset, page_token=values.unset,
             page_number=values.unset, page_size=values.unset):
        """
        Retrieve a single page of SettingsUpdateInstance records from the API.
        Request is executed immediately

        :param unicode sim: Filter the Settings Updates by Super SIM
        :param str page_token: PageToken provided by the API
        :param int page_number: Page Number, this value is simply for client state
        :param int page_size: Number of records to return, defaults to 50

        :returns: Page of SettingsUpdateInstance
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdatePage
        """
        data = values.of({'Sim': sim, 'PageToken': page_token, 'Page': page_number, 'PageSize': page_size, })

        response = self._version.page(method='GET', uri=self._uri, params=data, )

        return SettingsUpdatePage(self._version, response, self._solution)

    def get_page(self, target_url):
        """
        Retrieve a specific page of SettingsUpdateInstance records from the API.
        Request is executed immediately

        :param str target_url: API-generated URL for the requested results page

        :returns: Page of SettingsUpdateInstance
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdatePage
        """
        response = self._version.domain.twilio.request(
            'GET',
            target_url,
        )

        return SettingsUpdatePage(self._version, response, self._solution)

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SettingsUpdateList>'


class SettingsUpdatePage(Page):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    def __init__(self, version, response, solution):
        """
        Initialize the SettingsUpdatePage

        :param Version version: Version that contains the resource
        :param Response response: Response from the API

        :returns: twilio.rest.supersim.v1.settings_update.SettingsUpdatePage
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdatePage
        """
        super(SettingsUpdatePage, self).__init__(version, response)

        # Path Solution
        self._solution = solution

    def get_instance(self, payload):
        """
        Build an instance of SettingsUpdateInstance

        :param dict payload: Payload response from the API

        :returns: twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance
        """
        return SettingsUpdateInstance(self._version, payload, )

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SettingsUpdatePage>'


class SettingsUpdateInstance(InstanceResource):
    """ PLEASE NOTE that this class contains beta products that are subject to
    change. Use them with caution. """

    class Status(object):
        SCHEDULED = "scheduled"
        IN_PROGRESS = "in-progress"
        SUCCESSFUL = "successful"
        FAILED = "failed"

    def __init__(self, version, payload):
        """
        Initialize the SettingsUpdateInstance

        :returns: twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance
        :rtype: twilio.rest.supersim.v1.settings_update.SettingsUpdateInstance
        """
        super(SettingsUpdateInstance, self).__init__(version)

        # Marshaled Properties
        self._properties = {
            'sid': payload.get('sid'),
            'iccid': payload.get('iccid'),
            'sim_sid': payload.get('sim_sid'),
            'status': payload.get('status'),
            'packages': payload.get('packages'),
            'date_completed': deserialize.iso8601_datetime(payload.get('date_completed')),
            'date_created': deserialize.iso8601_datetime(payload.get('date_created')),
            'date_updated': deserialize.iso8601_datetime(payload.get('date_updated')),
        }

        # Context
        self._context = None
        self._solution = {}

    @property
    def sid(self):
        """
        :returns: The unique identifier of this Settings Update
        :rtype: unicode
        """
        return self._properties['sid']

    @property
    def iccid(self):
        """
        :returns: The ICCID associated with the SIM
        :rtype: unicode
        """
        return self._properties['iccid']

    @property
    def sim_sid(self):
        """
        :returns: The SID of the Super SIM to which this Settings Update was applied
        :rtype: unicode
        """
        return self._properties['sim_sid']

    @property
    def status(self):
        """
        :returns: The Status of this Settings Update
        :rtype: SettingsUpdateInstance.Status
        """
        return self._properties['status']

    @property
    def packages(self):
        """
        :returns: Array containing the different Settings Packages that will be applied to the SIM after the update completes
        :rtype: list[dict]
        """
        return self._properties['packages']

    @property
    def date_completed(self):
        """
        :returns: The time when the update successfully completed and the new settings were applied to the SIM
        :rtype: datetime
        """
        return self._properties['date_completed']

    @property
    def date_created(self):
        """
        :returns: The date this Settings Update was created
        :rtype: datetime
        """
        return self._properties['date_created']

    @property
    def date_updated(self):
        """
        :returns: The date this Settings Update was last updated
        :rtype: datetime
        """
        return self._properties['date_updated']

    def __repr__(self):
        """
        Provide a friendly representation

        :returns: Machine friendly representation
        :rtype: str
        """
        return '<Twilio.Supersim.V1.SettingsUpdateInstance>'
